# coding: utf-8

"""
    

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from xleap._client.models.chart_data import ChartData


class TestChartData(unittest.TestCase):
    """ChartData unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> ChartData:
        """Test ChartData
        include_option is a boolean, when False only required
        params are included, when True both required and
        optional params are included"""
        # uncomment below to create an instance of `ChartData`
        """
        model = ChartData()
        if include_optional:
            return ChartData(
                metric = '',
                values = [
                    1.337
                    ]
            )
        else:
            return ChartData(
                metric = '',
                values = [
                    1.337
                    ],
        )
        """

    def testChartData(self):
        """Test ChartData"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)


if __name__ == "__main__":
    unittest.main()
