# coding: utf-8

"""
    

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from xleap._client.models.exchange_token_request_user import ExchangeTokenRequestUser


class TestExchangeTokenRequestUser(unittest.TestCase):
    """ExchangeTokenRequestUser unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> ExchangeTokenRequestUser:
        """Test ExchangeTokenRequestUser
        include_option is a boolean, when False only required
        params are included, when True both required and
        optional params are included"""
        # uncomment below to create an instance of `ExchangeTokenRequestUser`
        """
        model = ExchangeTokenRequestUser()
        if include_optional:
            return ExchangeTokenRequestUser(
                username = 'Az',
                id = 56,
                first_name = '',
                last_name = '',
                date_joined = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f')
            )
        else:
            return ExchangeTokenRequestUser(
                username = 'Az',
        )
        """

    def testExchangeTokenRequestUser(self):
        """Test ExchangeTokenRequestUser"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)


if __name__ == "__main__":
    unittest.main()
