# coding: utf-8

"""
    

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from xleap._client.models.run import Run


class TestRun(unittest.TestCase):
    """Run unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> Run:
        """Test Run
        include_option is a boolean, when False only required
        params are included, when True both required and
        optional params are included"""
        # uncomment below to create an instance of `Run`
        """
        model = Run()
        if include_optional:
            return Run(
                id = '',
                answer = '',
                session_name = '',
                name = '',
                reference_example_id = '',
                start_time = '',
                end_time = '',
                extra = None,
                events = None,
                outputs = None,
                serialized = None,
                inputs = None,
                error = '',
                run_type = '',
                execution_order = -2147483648,
                child_execution_order = '',
                parent_run_id = ''
            )
        else:
            return Run(
                answer = '',
                start_time = '',
        )
        """

    def testRun(self):
        """Test Run"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)


if __name__ == "__main__":
    unittest.main()
