# coding: utf-8

"""
    

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from xleap._client.models.list_data_points200_response import ListDataPoints200Response


class TestListDataPoints200Response(unittest.TestCase):
    """ListDataPoints200Response unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> ListDataPoints200Response:
        """Test ListDataPoints200Response
        include_option is a boolean, when False only required
        params are included, when True both required and
        optional params are included"""
        # uncomment below to create an instance of `ListDataPoints200Response`
        """
        model = ListDataPoints200Response()
        if include_optional:
            return ListDataPoints200Response(
                next = '',
                previous = '',
                results = [
                    xleap._client.models.data_point.DataPoint(
                        id = '', 
                        question = '', 
                        answer = '', 
                        ground_truths = [
                            null
                            ], 
                        result = xleap._client.models.data_point_result.DataPoint_result(
                            results = xleap._client.models.results.results(), 
                            reasons = xleap._client.models.reasons.reasons(), ), 
                        contexts = [
                            ''
                            ], 
                        tags = xleap._client.models.tags.tags(), )
                    ]
            )
        else:
            return ListDataPoints200Response(
        )
        """

    def testListDataPoints200Response(self):
        """Test ListDataPoints200Response"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)


if __name__ == "__main__":
    unittest.main()
