# coding: utf-8

"""
    

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from xleap._client.api.v1_api import V1Api


class TestV1Api(unittest.TestCase):
    """V1Api unit test stubs"""

    def setUp(self) -> None:
        self.api = V1Api()

    def tearDown(self) -> None:
        pass

    def test_calculate_ai_scores_data_point(self) -> None:
        """Test case for calculate_ai_scores_data_point"""
        pass

    def test_create_api_key(self) -> None:
        """Test case for create_api_key"""
        pass

    def test_create_data_point_create(self) -> None:
        """Test case for create_data_point_create"""
        pass

    def test_create_metrics_data(self) -> None:
        """Test case for create_metrics_data"""
        pass

    def test_create_ml_model(self) -> None:
        """Test case for create_ml_model"""
        pass

    def test_create_org_list(self) -> None:
        """Test case for create_org_list"""
        pass

    def test_create_project_create(self) -> None:
        """Test case for create_project_create"""
        pass

    def test_create_prompt(self) -> None:
        """Test case for create_prompt"""
        pass

    def test_create_run(self) -> None:
        """Test case for create_run"""
        pass

    def test_destroy_data_point(self) -> None:
        """Test case for destroy_data_point"""
        pass

    def test_destroy_metrics_data(self) -> None:
        """Test case for destroy_metrics_data"""
        pass

    def test_destroy_ml_model(self) -> None:
        """Test case for destroy_ml_model"""
        pass

    def test_destroy_org_list(self) -> None:
        """Test case for destroy_org_list"""
        pass

    def test_destroy_prompt(self) -> None:
        """Test case for destroy_prompt"""
        pass

    def test_destroy_run(self) -> None:
        """Test case for destroy_run"""
        pass

    def test_exchange_token_exchange_token_request(self) -> None:
        """Test case for exchange_token_exchange_token_request"""
        pass

    def test_get_api_key_token_obtain(self) -> None:
        """Test case for get_api_key_token_obtain"""
        pass

    def test_get_user_org_member(self) -> None:
        """Test case for get_user_org_member"""
        pass

    def test_get_user_settings_user_preference(self) -> None:
        """Test case for get_user_settings_user_preference"""
        pass

    def test_list_api_keys(self) -> None:
        """Test case for list_api_keys"""
        pass

    def test_list_data_points(self) -> None:
        """Test case for list_data_points"""
        pass

    def test_list_metrics_datas(self) -> None:
        """Test case for list_metrics_datas"""
        pass

    def test_list_ml_models(self) -> None:
        """Test case for list_ml_models"""
        pass

    def test_list_org_lists(self) -> None:
        """Test case for list_org_lists"""
        pass

    def test_list_projects(self) -> None:
        """Test case for list_projects"""
        pass

    def test_list_prompts(self) -> None:
        """Test case for list_prompts"""
        pass

    def test_list_runs(self) -> None:
        """Test case for list_runs"""
        pass

    def test_partial_update_api_key_update(self) -> None:
        """Test case for partial_update_api_key_update"""
        pass

    def test_partial_update_data_point_create(self) -> None:
        """Test case for partial_update_data_point_create"""
        pass

    def test_partial_update_metrics_data(self) -> None:
        """Test case for partial_update_metrics_data"""
        pass

    def test_partial_update_ml_model(self) -> None:
        """Test case for partial_update_ml_model"""
        pass

    def test_partial_update_org_list(self) -> None:
        """Test case for partial_update_org_list"""
        pass

    def test_partial_update_project(self) -> None:
        """Test case for partial_update_project"""
        pass

    def test_partial_update_prompt(self) -> None:
        """Test case for partial_update_prompt"""
        pass

    def test_partial_update_run(self) -> None:
        """Test case for partial_update_run"""
        pass

    def test_retrieve_api_key(self) -> None:
        """Test case for retrieve_api_key"""
        pass

    def test_retrieve_chart_data(self) -> None:
        """Test case for retrieve_chart_data"""
        pass

    def test_retrieve_data_point(self) -> None:
        """Test case for retrieve_data_point"""
        pass

    def test_retrieve_metrics_data(self) -> None:
        """Test case for retrieve_metrics_data"""
        pass

    def test_retrieve_ml_model(self) -> None:
        """Test case for retrieve_ml_model"""
        pass

    def test_retrieve_org_detail(self) -> None:
        """Test case for retrieve_org_detail"""
        pass

    def test_retrieve_project_detail(self) -> None:
        """Test case for retrieve_project_detail"""
        pass

    def test_retrieve_prompt(self) -> None:
        """Test case for retrieve_prompt"""
        pass

    def test_retrieve_run(self) -> None:
        """Test case for retrieve_run"""
        pass

    def test_stat_project(self) -> None:
        """Test case for stat_project"""
        pass

    def test_update_api_key_update(self) -> None:
        """Test case for update_api_key_update"""
        pass

    def test_update_data_point_create(self) -> None:
        """Test case for update_data_point_create"""
        pass

    def test_update_metrics_data(self) -> None:
        """Test case for update_metrics_data"""
        pass

    def test_update_ml_model(self) -> None:
        """Test case for update_ml_model"""
        pass

    def test_update_org_list(self) -> None:
        """Test case for update_org_list"""
        pass

    def test_update_project(self) -> None:
        """Test case for update_project"""
        pass

    def test_update_prompt(self) -> None:
        """Test case for update_prompt"""
        pass

    def test_update_run(self) -> None:
        """Test case for update_run"""
        pass


if __name__ == "__main__":
    unittest.main()
