# coding: utf-8

"""
    

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from xleap._client.models.list_runs200_response import ListRuns200Response


class TestListRuns200Response(unittest.TestCase):
    """ListRuns200Response unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> ListRuns200Response:
        """Test ListRuns200Response
        include_option is a boolean, when False only required
        params are included, when True both required and
        optional params are included"""
        # uncomment below to create an instance of `ListRuns200Response`
        """
        model = ListRuns200Response()
        if include_optional:
            return ListRuns200Response(
                next = '',
                previous = '',
                results = [
                    xleap._client.models.run.Run(
                        id = '', 
                        answer = '', 
                        session_name = '', 
                        name = '', 
                        reference_example_id = '', 
                        start_time = '', 
                        end_time = '', 
                        extra = xleap._client.models.extra.extra(), 
                        events = xleap._client.models.events.events(), 
                        outputs = xleap._client.models.outputs.outputs(), 
                        serialized = xleap._client.models.serialized.serialized(), 
                        inputs = xleap._client.models.inputs.inputs(), 
                        error = '', 
                        run_type = '', 
                        execution_order = -2147483648, 
                        child_execution_order = '', 
                        parent_run_id = '', )
                    ]
            )
        else:
            return ListRuns200Response(
        )
        """

    def testListRuns200Response(self):
        """Test ListRuns200Response"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)


if __name__ == "__main__":
    unittest.main()
